@@ -3,6 +3,7 @@ from __future__ import print_function
 from __future__ import division
 
 import tensorflow as tf
+tf.get_logger().setLevel('ERROR')
 tf.compat.v1.disable_eager_execution()
 
 from . import constraints
@@ -18,6 +19,7 @@ from .models.model import SciModel
 from .constraints import Constraint, Data, Tie
 
 # Also importable from root
+from .utils.math import *
 from .utils import math
 from .utils.utilities import reset_session, clear_session
 from .utils.utilities import set_random_seed
@@ -30,10 +32,10 @@ __copyright__ = "Copyright 2019, Physics-Informed Deep Learning"
 __credits__ = []
 __url__ = "http://github.com/sciann/sciann]"
 __license__ = "MIT"
-__version__ = "0.5.0.1"
+__version__ = "0.5.3"
 __cite__ = \
     '@misc{haghighat2019sciann, \n' +\
-    '    title={SciANN: A Keras wrapper for scientific computations and physics-informed deep learning using artificial neural networks}, \n' +\
+    '    title={SciANN: A Keras/Tensorflow wrapper for scientific computations and physics-informed deep learning using artificial neural networks}, \n' +\
     '    author={Ehsan Haghighat and Ruben Juanes}, \n' +\
     '    year={2020}, \n' +\
     '    eprint={2005.08803}, \n' +\
