@@ -7,15 +7,18 @@ from __future__ import print_function
 
 import os
 
+import tensorflow as tf
 import tensorflow.python.keras.backend as K
 import tensorflow.python.keras as k
 import numpy as np
 
 from tensorflow.python.keras.models import Model
 from tensorflow.python.keras.utils.vis_utils import plot_model
+from tensorflow import gradients as tf_gradients
 
 from ..utils import unpack_singleton, to_list
 from ..utils import is_variable, is_constraint, is_functional
+from ..utils.optimizers import GradientObserver, ScipyOptimizer
 
 from ..functionals import Variable
 from ..functionals import RadialBasis
@@ -124,10 +127,20 @@ class SciModel(object):
             **kwargs
         )
         # compile the model.
-        model.compile(
-            loss=loss_func,
-            optimizer=optimizer
-        )
+        loss_weights = [K.variable(1.0) for v in output_vars]
+        if isinstance(optimizer, str) and \
+                len(optimizer.lower().split("scipy-")) > 1:
+            model.compile(
+                loss=loss_func,
+                optimizer=GradientObserver(method=optimizer),
+                loss_weights=loss_weights
+            )
+        else:
+            model.compile(
+                loss=loss_func,
+                optimizer=optimizer,
+                loss_weights=loss_weights
+            )
         # model.train_function = True
 
         # set initial state of the model.
@@ -157,6 +170,12 @@ class SciModel(object):
     def inputs(self):
         return self._inputs
 
+    def load_weights(self, file):
+        if os.path.exists(file):
+            self.model.load_weights(file)
+        else:
+            raise ValueError('File not found.')
+
     def verify_update_constraints(self, constraints):
         ver = []
         for old, new in zip(self._constraints, constraints):
@@ -196,11 +215,12 @@ class SciModel(object):
               batch_size=2**6,
               epochs=100,
               learning_rate=0.001,
+              adaptive_weights=False,
               shuffle=True,
               callbacks=None,
               stop_lr_value=1e-8,
               reduce_lr_after=None,
-              reduce_lr_min_delta=0.001,
+              reduce_lr_min_delta=0.,
               stop_after=None,
               stop_loss_value=1e-8,
               save_weights_to=None,
@@ -236,6 +256,8 @@ class SciModel(object):
                     learning_rate = ([0, 100, 1000], [0.001, 0.0005, 0.00001])
             shuffle: Boolean (whether to shuffle the training data).
                 Default value is True.
+            adaptive_weights: Defaulted to False (no updates - evaluated once in the beginning).
+                Used if the model is compiled with adaptive_weights.
             callbacks: List of `keras.callbacks.Callback` instances.
             reduce_lr_after: patience to reduce learning rate or stop after certain missed epochs.
                 Defaulted to epochs max(10, epochs/10).
@@ -296,7 +318,7 @@ class SciModel(object):
                                           patience=stop_after, min_delta=1e-9),
                 k.callbacks.TerminateOnNaN(),
                 EarlyStoppingByLossVal(stop_loss_value),
-                EarlyStoppingByLearningRate(stop_lr_value)
+                EarlyStoppingByLearningRate(stop_lr_value),
             ]
         # prepare X,Y data.
         x_true = to_list(x_true)
@@ -358,6 +380,9 @@ class SciModel(object):
             else:
                 for i, cw in enumerate(target_weights):
                     sample_weights[i] *= cw
+        else:
+            target_weights = len(y_true) * [1.0]
+
         # save model.
         model_file_path = None
         if save_weights_to is not None:
@@ -373,10 +398,26 @@ class SciModel(object):
                 print("\nWARNING: Failed to save model.weights to the provided path: {}\n".format(save_weights_to))
         if model_file_path is not None:
             callbacks.append(model_check_point)
-        # perform the training.
-        history = self._model.fit(
+
+        if isinstance(self._model.optimizer, GradientObserver):
+            opt = ScipyOptimizer(self._model)
+            opt_fit_func = opt.fit
+        else:
+            opt_fit_func = self._model.fit
+
+        if adaptive_weights:
+            callbacks.append(
+                GradientPathologyLossWeight(
+                    self._model, x_true, y_star, sample_weights,
+                    beta=0.8, freq=adaptive_weights
+                ),
+                # k.callbacks.CSVLogger('GP.log')
+            )
+
+        # training the models.
+        history = opt_fit_func(
             x_true, y_star,
-            sample_weight=sample_weights,  #sums to number of samples.
+            sample_weight=sample_weights,  # sums to number of samples.
             epochs=epochs,
             batch_size=batch_size,
             shuffle=shuffle,
@@ -384,11 +425,13 @@ class SciModel(object):
             validation_data=validation_data,
             **kwargs
         )
+
         if save_weights_to is not None:
             try:
                 self._model.save_weights("{}-end.hdf5".format(save_weights_to))
             except:
                 print("\nWARNING: Failed to save model.weights to the provided path: {}\n".format(save_weights_to))
+
         # return the history.
         return history
 
@@ -611,3 +654,69 @@ class EarlyStoppingByLearningRate(k.callbacks.Callback):
             print("Revise 'stop_lr_value={:0.12f}' in '.train' if it was not your intent. ".format(self.value))
 
 
+class GradientPathologyLossWeight(k.callbacks.Callback):
+    def __init__(self, model, inputs, targets, weights, beta=0.8, freq=100):
+        super(k.callbacks.Callback, self).__init__()
+        self.inputs = inputs
+        self.targets = targets
+        self.weights = weights
+        self.loss_grads = []
+        for i in range(len(model.outputs)):
+            yp = model.outputs[i]
+            ys = targets[i]
+            ws = weights[i]
+            f = tf.reduce_mean(
+                model.loss_functions[i](ys, yp, sample_weight=ws)
+            )
+            gf = tf_gradients(f, model.trainable_weights, unconnected_gradients='zero')
+            self.loss_grads.append(
+                K.function(model.inputs, gf)
+            )
+        self.freq = 0 if isinstance(freq, bool) else freq
+        self.beta = beta
+
+    def on_train_begin(self, logs={}):
+        # update loss-weights
+        self.update_loss_weights(0)
+
+    def on_epoch_end(self, epoch, logs={}):
+        if epoch > 1 and self.freq > 1 \
+                and epoch % self.freq == 0:
+            self.update_loss_weights(epoch)
+        # log the weights in the history output.
+        logs['adaptive_weights'] = [
+            K.get_value(wi) for wi in self.model.loss_weights
+        ]
+
+    def eval_loss_weights(self):
+        # eval new gradients
+        updated_grads = []
+        for lgi, trg in zip(self.loss_grads, self.targets):
+            updated_grads.append(
+                np.concatenate(
+                    [np.abs(wg).flatten() for wg in lgi(self.inputs)]
+                )
+            )
+        # eval max normalization on PDE.
+        ref_grad = updated_grads[0].max()
+        # mean loss terms
+        mean_grad = []
+        for ws in updated_grads:
+            ws_mean = ws.mean()
+            mean_grad += [1.0 if ws_mean == 0. else ws_mean]
+        return (ref_grad, mean_grad)
+
+    def update_loss_weights(self, epoch):
+        ref_grad, mean_grad = self.eval_loss_weights()
+        # evaluate new weights
+        # new weights
+        for i, wi in enumerate(self.model.loss_weights):
+            if i > 0:
+                # gp_weights[i]
+                gp_weight = ref_grad / (mean_grad[i] * K.get_value(wi))
+                new_val = (1.0-self.beta)*K.get_value(wi) + self.beta*gp_weight
+                K.set_value(self.model.loss_weights[i], new_val)
+        # print updates
+        print('adaptive_weights at epoch {}:'.format(epoch),
+              [K.get_value(wi) for wi in self.model.loss_weights])
+
